package net.sourceforge.basher.yourkit;

import com.yourkit.Constants;
import com.yourkit.api.*;
import net.sourceforge.basher.*;
import net.sourceforge.basher.events.*;
import org.apache.commons.logging.Log;

/**
 * @author Johan Lindquist
 */
public class YourKitController implements BasherEventListener
{
    private Controller _controller;
    private Log _log;

    private String _host = "localhost";
    private int _port = 10020;

    private String _filters = "com.sun.*\\njava.*\\njavax.*\\nsun.*\\njrockit.*\\nnet.sourceforge.basher.*";

    public void setHost(final String host)
    {
        _host = host;
    }

    public void setPort(final int port)
    {
        _port = port;
    }

    public void setLog(final Log log)
    {
        _log = log;
    }

    public void initializeService() throws Exception
    {
        _log.info("Initializing YourKit Controller");
        _log.info("Hostname: " + _host);
        _log.info("Port: " + _port);

        connect();
    }

    public void basherEvent(final BasherEvent basherEvent)
    {
        if (basherEvent instanceof CollectionStartedEvent)
        {
            // Only do profiling
            try
            {
                BasherContext basherContext = getBasherContext(basherEvent);

                YourKitProfilerContext yourKitProfilerContext = extractYourkitProfilerContext(basherContext);
                if (yourKitProfilerContext == null)
                {
                    return;
                }

                if (_controller == null)
                {
                    _log.warn("Not connected to profiler, will not perform any action");
                    return;
                }

                if (yourKitProfilerContext.isGarbageCollectBeforeStart())
                {
                    _log.info("Forcing GC before starting profiling");
                    // Force a GC
                    final long[] longs = _controller.forceGC();
                    _log.info("GC completed");
                    _log.info("Memory Before: " + longs[0]);
                    _log.info("Memory After: " + longs[1]);
                    _log.info("Total Reclaimed: " + (longs[0] - longs[1]));

                }

                // Start profiling
                // Need he flags here
                _controller.startCPUProfiling(ProfilingModes.CPU_TRACING, _filters);

            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
        if (basherEvent instanceof CollectionStoppedEvent)
        {
            BasherContext basherContext = getBasherContext(basherEvent);

            // Only do profiling
            try
            {
                YourKitProfilerContext yourKitProfilerContext = extractYourkitProfilerContext(basherContext);
                if (yourKitProfilerContext == null)
                {
                    return;
                }

                if (_controller == null)
                {
                    _log.warn("Not connected to profiler, will not perform any action");
                    return;
                }
                
                if (yourKitProfilerContext.isGarbageCollectBeforeStart())
                {
                    _log.info("Forcing GC before capturing snapshot");
                    // Force a GC
                    final long[] longs = _controller.forceGC();
                    _log.info("GC completed");
                    _log.info("Memory Before: " + longs[0]);
                    _log.info("Memory After: " + longs[1]);
                    _log.info("Total Reclaimed: " + (longs[0] - longs[1]));

                }

                // Stop - capture snapshot
                _log.info("Capturing snapshot");
                final String s = _controller.captureSnapshot(ProfilingModes.SNAPSHOT_WITHOUT_HEAP);
                _log.info("Snapshot captured to: " + s);

                _log.info("Stopping profiling");
                _controller.stopCPUProfiling();
                _log.info("Profiling stopped");

            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }

    }

    private YourKitProfilerContext extractYourkitProfilerContext(final BasherContext basherContext)
    {
        final ProfilerContext profilerContext = basherContext.getProfilerContext();
        if (profilerContext == null)
        {
            _log.info("No profiler context defined in Basher context, will do nothing");
            return null;
        }
        if (!(profilerContext instanceof YourKitProfilerContext))
        {
            _log.info("Profiler context not applicable to YourKit, will do nothing");
            return null;
        }

        return (YourKitProfilerContext) profilerContext;
    }

    private void connect()
    {
        try
        {
            _log.info("Connecting to profiling agent");
            _controller = new Controller(_host, _port);
            _log.info("Successfully connected to profiling agent");
        }
        catch (Exception e)
        {
            _log.error("Error while setting up YourKit controller: " + e.getMessage(), e);
        }
    }

    private void disconnect(final BasherContext basherContext)
    {
        if (_controller == null)
        {
            _log.warn("No YourKit controller available, will do nothing");
        }
        _controller = null;
    }

    private BasherContext getBasherContext(final BasherEvent basherEvent)
    {
        return ((CollectionEvent) basherEvent).getBasherContext();
    }
}
